# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2001,2002,2003 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _awkassign.awk,v 1.2 2006/01/10 17:28:33 carlo Exp $
# =====================================================================
# boolean _awkassign(string name, array value \
#			[, boolean null, boolean begin, string ofile])
#
# Turn the 'value' array into a valid AWK array assignment statement.
# The No. of elements
# is expected in value[0]. The resulting assignment is then printed to
# stdout, properly escaped for AWK. Unless 'null' is non-zero, empty
# input values will result in a "delete" statement on output.
# If 'begin' is non-zero, the resulting assignment is done inside a
# "BEGIN{}" block.
# =====================================================================

function _awkassign(name,value,null,begin,ofile,		i) {

  i=0
  if (ofile == _NULL) ofile = _STDOUT

  # Check for legal name.
  if (name !~ /^_*[A-Za-z][A-Za-z0-9_]*$/) return _EINVAL

  if (begin) print "BEGIN {" >> ofile

  if (value[0] <= 0) {			# Handle special case.
     if (!null) printf("delete %s\n", name) >> ofile
     else printf("%s=\"\"", name) >> ofile
     if (begin) print "}" >> ofile
     return 0
  }

  printf("%s[0]=%d\n", name, value[0]) >> ofile

  while (++i <= value[0]) {
     printf("%s[%d]=%s\n", name, i, _escape(value[i])) >> ofile
  }

  if (begin) print "}" >> ofile
}

