# =====================================================================
# AWK functions for CSA.
#
# Copyright (c) 2004 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License 
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================
# $Id: _response.awk,v 1.1 2006/01/10 17:28:33 carlo Exp $
# =====================================================================
# void _response(string name, string value,\
#				int argnum, int argpos, boolean noenc)
#
# Prepare encoded versions of the specified name/value pair, for use in
# response templates. If 'noenc' is non-zero, then 'value' is not
# encoded and is loaded verbatim into ENVIRON[]. This is mainly meant
# to allow for the inclusion of literal [X]HTML fragments.
# The 'argnum' and 'argpos' parameters are used to produce the CSA-RPC
# response, if needed. See also '_tbltordf()'.
# =====================================================================

function _response(name,value,argnum,argpos,noenc,	i,a,nf) {


  # Set URI-, URP-, ISO- and XML-PCDATA- encodings in ENVIRON[]. I
  # need to prepare all three encodings because this function is used
  # for both HTML and XML templates. Note that if the string is to be
  # included in an XML document and it must represent a URI, then the
  # the URI_ or URP_ versions, whichever applies, should be used. ISO_
  # encoding is practically only useful with HTML, and it is of no use
  # with XML. Literal, unencoded values are provided in TXT- entries,
  # which are therefore equivalent to specifying the 'noenc' option.
  # The latter however is still necessary for backward compatibility.

  # All these pre-encoded values are still created for backward
  # compatibility, but given the new encoding flags they are no
  # longer necessary.

  if (name != _NULL) {
     if (noenc == _O_NOENC) ENVIRON[name] = value
     else {
	ENVIRON["URI_" name] = _uriencode(value)
	ENVIRON["URP_" name] = _uriencode(value,1)
	ENVIRON["ISO_" name] = _isoencode(value)
	ENVIRON["XML_" name] = _xmlencode(value)
	ENVIRON["TXT_" name] = value
     }
  }

  if (argnum && argpos) {
     if (argnum > _OUT[0]) _OUT[0] = argnum
     nf = split(_OUT[argnum],a,"\t")
     if (argpos > nf) nf = argpos
     a[argpos] = value
     gsub(/\t/,"\\t",a[argpos])
     gsub(/\n/,"\\n",a[argpos])
     _OUT[argnum] = a[1]
     for (i=2; i<=nf; i++) _OUT[argnum] = _OUT[argnum] "\t" a[i]
  }
}

