#!/usr/bin/rc -p
# *********************************************************************
# CSA: CGI Shell Architecture Remote Procedure Call processor.
#
# Copyright (c) 2002,2003,2004,2005 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# *********************************************************************
# $Id: CSA.cgi,v 1.1 2006/01/10 17:28:33 carlo Exp $

# Hard-coded CSA settings (don't touch!).

CSA_VERSION='@CSA_VERSION@'
CSA_INSTALL='@CSA_INSTALL@'

# *********************************************************************
#                Start of user-modifiable settings
# *********************************************************************

# Set this to '1' to trace really everything, see below.
CSA_DUMP = 0

# Change these settings as appropriate and then copy this script
# to your HTTP daemon /cgi-bin/ directory. If you are using per-user
# /cgi-bin/ areas (usually with the Apache Web Server SuEXEC or with
# the cgiwrap utility), then act as appropriate, according to the
# relevant documentation of said techniques.

logname = jsmith
home = /home/$logname
libstem = $home/.csa
tmpdir = /tmp
path = ($CSA_INSTALL/bin /usr/local/bin /usr/bin /bin)

#CSA_RPC_MAXSIZE=100000
#CSA_AWK_NEWCACHE=1	     # uncomment to always force cache rebuild.

# *********************************************************************
#                 End of user-modifiable settings
# *********************************************************************

if (~ $CSA_DUMP 1 && !~ $CSA_DUMP_FILE /*) {
   CSA_DUMP_FILE = `{mktemp /tmp/csa.stderr.$pid.XXXXXX}
   ~ $bqstatus 0 || exit 1
   CSA_STDERR = $CSA_DUMP_FILE
   NOSQL_STDERR = $CSA_DUMP_FILE
   NOSQL_DEBUG = 1
   CSA_ENV_FILE = `{mktemp /tmp/csa.env.$pid.XXXXXX}
   ~ $bqstatus 0 || exit 1
   rc -p -c env > $CSA_ENV_FILE
   CSA_ARGS_FILE = `{mktemp /tmp/csa.args.$pid.XXXXXX}
   ~ $bqstatus 0 || exit 1
   echo $* > $CSA_ARGS_FILE
   if (~ $REQUEST_METHOD POST PUT) {
      CSA_INPUT_FILE = `{mktemp /tmp/csa.stdin.$pid.XXXXXX}
      ~ $bqstatus 0 || exit 1
      cat > $CSA_INPUT_FILE
      # append mandatory, or rc(1) will also dump garbage into the file!
      exec rc -p -x $0 $* < $CSA_INPUT_FILE >>[2]$CSA_DUMP_FILE
   }  else exec rc -p -x $0 $* >>[2]$CSA_DUMP_FILE
}

# needed by cgi.awk.
if (~ $PATH_INFO ()) {
   CSA_REQUEST_URI = /UNKNOWN
}  else CSA_REQUEST_URI = $PATH_INFO

if (~ $QUERY_STRING) {
   CSA_REQUEST_URI = $CSA_REQUEST_URI'?UNKNOWN'
}  else CSA_REQUEST_URI = $CSA_REQUEST_URI'?'$QUERY_STRING

# Misc. defs

if (!~ $CSA_DUMP_FILE /*) {
   CSA_DEBUG=0					# Must be not-null.
}  else CSA_DEBUG=1

# The actual temporary directory will *not* be publicly writable !!
TMPDIR = $tmpdir/$logname-csa-$CSA_VERSION

nl='
'
noinput=()

# *********************************************************************
#			Local functions
#
# Note: to save environment space, functions have the same name as the
# actual CSA library ones. This may not matter if rc(1) subprograms
# are called with flag '-p', but it definitely does matter if they are
# included with ``. file''.
# *********************************************************************

# Define a few dummy functions that may be called by this script before
# their real versions have been loaded.

fn csaDebug csaAuth.send csaSetTime csaUnlist csaUniq {return 0}

fn csaCleanup {
   ~ $CSA_DEBUG 0 || csaDebug $0 $*

   #csaUnlist CSA_WORKFILES $CSA_RPC_CMD $CSA_RPC_AWK \
   #					$CSA_RPC_PUT $CSA_RDF_ARG

   # Prevent wasting time trying to remove the same files twice,
   # and remove any reference to /dev/null, to prevent wasteful
   # cluttering of stderr.log. Note: CSA_LOCKS must be listed
   # *after* CSA_WORKFILES.

   csa1 = ($CSA_RPC_CMD $CSA_RPC_AWK $CSA_RPC_PUT
	   $CSA_RDF_ARG $CSA_WORKFILES $CSA_LOCKS $CSA_CGI_STDIN
	   $TMPDIR/fncache$pid.tmp $TMPDIR/hexBinary.$pid.*) {

	csaUniq csa1; csaUnlist csa1 /dev/null; rm -rf -- $csa1
   }
}

fn csaExit {
   # Preserve workfiles if requested. This must be handled first,
   # even before csaDebug is called.
   if (~ $1 -n --no-cleanup) {
      CSA_WORKFILES = ()
      shift
   }
   ~ $CSA_DEBUG 0 || csaDebug $0 $*
   csaCleanup
   exit $1
}

fn csaHttp.header {
   ~ $CSA_DEBUG 0 || csaDebug $0 $*
   csa1=() csa2=() {
     while (~ $1 -*) {
	switch ($1) {
	  case -e --end-of-header
	       csa1=$nl
	  case -d --deferred
	       csa2=1
	}
	shift
     }
     ~ $#* 2 || return 0			# nothing to do.
     ~ $1 [Ee]xpires && ~ $2 -1 &&
	  *=($1 'Thu, 01 Jan 1970 00:00:00 +0000')
     if (~ $csa2 ()) {
     	echo $1:' '$2$csa1
     }	else {
	~ $CSA_HTTP_HEADERS () || CSA_HTTP_HEADERS=$CSA_HTTP_HEADERS$nl
	CSA_HTTP_HEADERS=$CSA_HTTP_HEADERS$1:' '$2$csa1
     }
   }
}

# *********************************************************************
# HTTP/1.1 Status Codes:
#
# 100-199: Codes in the 100s are informational, indicating that the
#	   client should respond with some other action.
#
# 200-299: Values in the 200s signify that the request was successful.
#
# 300-399: Values in the 300s are used for files that have moved
#	   and usually include a Location header indicating the new
#	   address.
#
# 400-499: Values in the 400s indicate an error by the client.
#
# 500-599: Codes in the 500s signify an error by the server.
#
# See http://www.informit.com/articles/article.asp?p=29817&seqNum=7
# (among other places) for more information.
# *********************************************************************

fn csaHttp.status {
   ~ $CSA_DEBUG 0 || csaDebug $0 $*

   csa1=() {

     # Add more NPH auto-detection here.
     switch ($SCRIPT_NAME) {
       case /cgi-bin/nph-*
	    csa1 = nph
     }

     ~ $1 [1-5][0-9][0-9] || * = ()
     switch ($1) {
       case 200; * = (7  $1 OK)
       case 201; * = (12 $1 Created)
       case 202; * = (13 $1 Accepted)
       case 204; * = (0  $1 No Content)
       case 301; * = (22 $1 Moved Permanently)
       case 302; * = (22 $1 Moved Temporarily)
       case 303; * = (14 $1 See Other)
       case 400; * = (16 $1 Bad Request)
       case 401; * = (17 $1 Unauthorized)
       case 403; * = (14 $1 Forbidden)
       case 404; * = (14 $1 Not Found)
       case 405; * = (23 $1 Method Not Allowed)
       case 406; * = (19 $1 Not Acceptable)
       case 409; * = (13 $1 Conflict)
       case 413; * = (29 $1 Request Entity Too Large)
       case 500; * = (26 $1 Internal Server Error)
       case 501; * = (20 $1 Not Implemented)
       case 503; * = (24 $1 Service Unavailable)
       case *;   * = (23 550 CSA Internal Error)	# Unknown status code.
     }

     CSA_HTTP_LENGTH = $1; shift
     CSA_HTTP_STATUS = $*

     if (~ $csa1 ()) {
        csaHttp.header Status $^CSA_HTTP_STATUS
     }	else {
	echo HTTP/1.0 $^CSA_HTTP_STATUS
	csaHttp.header Date $CSA_TIME_HTTP
	csaHttp.header Server $SERVER_SOFTWARE
	csaHttp.header X-Parsed no		# useful for debugging.
     }
   }
}

fn csaExit.http {
   ~ $CSA_DEBUG 0 || csaDebug $0 $*
   csaCleanup
   csaHttp.status $1

   # Experiments have shown that it is preferable not to forbid
   # caching on the part of the client, but rather tell the
   # latter that the document has changed. 

   csaHttp.header Last-Modified $CSA_TIME_HTTP

   # The following cache-related directives do more harm than good,
   # so don't use them by default.
   #
   #csaHttp.header Pragma no-cache			# for HTTP/1.0
   #csaHttp.header Cache-control no-cache		# for HTTP/1.1
   #csaHttp.header Expires -1

   csaHttp.header X-Powered-By CSA/$CSA_VERSION
   csaAuth.send					# mandatory!

   ~ $CSA_HTTP_LENGTH 0 || csaHttp.header Content-Length $CSA_HTTP_LENGTH

   # Make picky clients happy.
   csaHttp.header --end-of-header Content-Type text/plain

   # Add more "silent" codes if necessary.
   ~ $CSA_HTTP_STATUS(1) 204 || echo $^CSA_HTTP_STATUS
   exit
}

fn csaExit.fault {
   # Preserve workfiles if requested. This must be handled first,
   # even before csaDebug is called.
   if (~ $1 -n --no-cleanup) {
      CSA_WORKFILES = ()
      shift
   }
   ~ $CSA_DEBUG 0 || csaDebug $0 $*
   ~ $1 [45][0-9][0-9] || * = 550
   csaExit.http $1
}

fn csaExit.ok { ~ $CSA_DEBUG 0 || csaDebug $0 $*; csaExit.http 204 }

# *********************************************************************
#			   Main program
# *********************************************************************

# Set time stuff, leaving any ISINDEX argument alone.

* = () {
  * = `{date '+%Y %m %d %H %M %S %Z %a %b %s %z'}

  # Save values for possible later use by application programs
  # to built custom local date formats.
  CSA_TIME_LOCAL = $*

  CSA_TIME_ISO = $1-$2-$3' '$4:$5:$6

  CSA_TIME_LOG = $CSA_TIME_ISO.$7

  ~ $7 UTC GMT && CSA_TIME_ISO = $CSA_TIME_ISO^Z

  # A second date(1) run is necessary to set UTC time values.
  * = `{date -u '+%Y %m %d %H %M %S %Z %a %b %s %z'}

  # Save values for possible later use by application programs
  # to built custom UTC date formats.
  CSA_TIME_UTC = $*

  CSA_TIME_HTTP = $8,' '$3' '$9' '$1' '$4:$5:$6' 'GMT
}

# Set the program name. If more special names are added remember to add
# them also to csaMainlib.rc.

switch ($0) {
  case *CSAI *CSAI.* *CSAI-*
       CSA_PGM = CSAI
  case *
       CSA_PGM = CSA				# default
}

CSA_ISINDEX=()					# Just in case

# Use special care on TMPDIR.
# The following mkdir+chmod sequence is subject to race conditions, but
# that isn't a problem, as it doesn't matter which process is doing
# what, provided that the directory is succesfully created.

if (~ $TMPDIR^* *'*') {
   mkdir $TMPDIR && chmod 700 $TMPDIR || csaExit.fault 550
}

# Perform some basic checks. To date, the CSA non-standard PUT and
# DELETE are supported, while their real HTTP counterparts are
# theoretically supported but have not been tested yet. Standard Web
# browsers generally only support GET/POST but programmatic clients may
# also use PUT/DELETE. Although DELETE does not require any content
# and could therefore be dispatched via GET, it would violate GET's
# idempotency property, thus POST is used instead (although possibly
# with Content-Length=0). The only form of QUERY_STRING supported by
# CSA in REST-mode is ISINDEX, therefore any other GET arguments must
# be specified in the form of PATH_INFO positional parameters. That is,
# rather than using "http://my.server/search?q=foo+bar", we will have
# to use something "http://my.server/search/foo+bar". Likewise, to
# specify that we want only from result item no. 10 onwards, we can use:
# "http://my.server/search/foo+bar/10".

switch ($REQUEST_METHOD) {
  case GET HEAD DELETE PUT;
  case POST
       if (~ $1 PUT DELETE) {
	  REQUEST_METHOD = $1
	  # Remove the REST verb, to prevent it from being
	  # treated as an argument by underlying CSA layers.
	  ~ $QUERY_STRING $REQUEST_METHOD && QUERY_STRING = ()
       }
  case *
       csaExit.fault 405
}

# Allocate the necessary work-files.

CSA_RPC_CMD = $TMPDIR/rpc$pid.tmp
CSA_RPC_AWK = $TMPDIR/awk$pid.tmp
CSA_RDF_ARG = $TMPDIR/arg$pid.tmp
CSA_RPC_PUT = $TMPDIR/put$pid.tmp
CSA_CGI_STDIN = $TMPDIR/stdin$pid.tmp

# Note: opening stdin with no input causes some setups to hang.
# This happens for instance with thttpd/2.x.

~ $CONTENT_LENGTH [1-9]* || noinput = 1

# use temporary AWK function cache if main file is missing.

if (~ $TMPDIR/fncache.awk* *'*' || ~ $CSA_AWK_NEWCACHE 1) {
   cat $CSA_INSTALL/lib/include/*.awk > $TMPDIR/fncache$pid.tmp
   cmd = (mawk -f$TMPDIR/fncache$pid.tmp)
}  else {
   cmd = (mawk -f$TMPDIR/fncache.awk)
}

cmd = ($cmd -f^($CSA_INSTALL/lib/csa-^((iso-8859-1 rpciolib)^.awk))
			-f$CSA_INSTALL/lib/csaCgi.awk '''-v_PID='''$pid)

# Note the use of 'csa-escape -z' below. We need to choose an escaping
# mechanism that is unlikely to apply to part headers of multipart input.
# That is, we cannot use quoted-printable escaping, as '=' signs appare
# in most MIME defs in the input stream, and we would then need to un-escape
# them during multipart processing in cgi.awk.
# 
# There is a bug in MSIE 5.0x, whereby the browser does not include the
# trailing CR+LF in the content-length value that is supplied to the
# server. Because of this, the CGI program needs to read no more than
# 'content-length' bytes and then close the stdin stream. If this is    
# not done, the server hangs and so does the client, until eventually   
# the web server times-out and closes the connection, thus causing the
# operation to fail.
# This happens with thttpd/2.24, I have not tested it with other
# servers. I handle this by inserting a call to 'csa-escape -z' also  
# with the 'application/x-www-form-urlencoded' input type, not because 
# input needs to be scanned for binary zeroes, as there cannot be any,
# but just to fix the bogus content-length value supplied by MSIE. This
# has also the beneficial side-effect of dropping the trailing CR+LF
# supplied by MSIE, which is not part of the actual data.

switch ($CONTENT_TYPE) {
  case ();	# Void action, also for the HEAD/DELETE requests.

  case application/x-www-form-urlencoded multipart/form-data*
       ~ $noinput () && cmd = (csa-escape -z '|' $cmd)

  case *
       # Any other MIME types can be accepted only with PUT.
       ~ $REQUEST_METHOD PUT || csaExit.fault 406
       ~ $noinput () && cmd = (csa-escape -z '|' $cmd)
}

if (~ $noinput ()) {
   cmd = (tee $CSA_CGI_STDIN '|' $cmd)
}  else {
   touch $CSA_CGI_STDIN				# must exist!
   cmd = ($cmd -- /dev/null)
}

eval $cmd || csaExit.fault

# Keep the environment as small as possible.
logname = ()
libstem = ()
noinput = ()
tmpdir = ()

. $CSA_RPC_CMD

csaExit $status					# This won't hurt.

# EOF
