/* Written by and copyright Carlo Strozzi <carlos@linux.it>. */

/* csa-cpmv: copy and then rename files. This is a trivial all-in-one
   replacemente for the cp+mv command sequence. It is meant to
   spare processes in CSA commit operations.

   Copyright (c) 2001,2002,2003 Carlo Strozzi

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


   Usage:  csa-cpmv <from> <tmp> <to> [<from> <tmp> <to> ...]

   Examples:

   $ csa-cpmv file1 file2 file3

   This will copy file1 to file2 and then rename file2 into file3.

   Files must be specified in triplets, or the program will complain.

*/

static char Psz_RCS[] = "$Date: 2004/08/08 21:15:38 $";

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

int main(int argc, char **argv) {

	struct stat statbuf;
	int f1, f2, n, i;
	mode_t perms;
	div_t arglist;
	char buf[BUFSIZ];

	/* check that args are in triplets */

	arglist = div(argc-1,3);

	if (arglist.quot < 1 || arglist.rem > 0) {
		fputs("Usage: csa-cpmv <from> <tmp> <to> ...\n", stderr);
		exit(1);
	}

	/* process one triplet at a time */
	for (i=1; i<argc; i+=3) {

	    if (stat(argv[i], &statbuf) < 0) {
		fprintf(stderr, "csa-cpmv: can't stat %s\n", argv[i]);
		exit(1);
	    }

	    if ((f1 = open(argv[i], O_RDONLY)) < 0) {
		fprintf(stderr, "csa-cpmv: can't open %s\n", argv[i]);
		exit(1);
	    }

	    perms = statbuf.st_mode & 00007777;

	    if ((f2 = creat(argv[i+1], perms)) < 0) {
		fprintf(stderr, "csa-cpmv: can't create %s\n", argv[i+1]);
		exit(1);
	    }

	    while ((n = read(f1, buf, BUFSIZ)) > 0)
		if (write(f2, buf, n) != n) {
			fputs("csa-cpmv: write error", stderr);
			exit(1);
		}

	    if (rename(argv[i+1], argv[i+2]) < 0) {
		fprintf(stderr, "csa-cpmv: %s -> %s failed\n",argv[i+1],argv[i+2]);
		exit(1);
	    }

	    if (close(f1) < 0) {
		fprintf(stderr, "csa-cpmv: close error on %s\n",argv[i]);
		exit(1);
	    }

	    if (close(f2) < 0) {
		fprintf(stderr, "csa-cpmv: close error on %s\n",argv[i+2]);
		exit(1);
	    }
	}

	exit(0);

}

/* EOF */
